@@ -56,13 +56,18 @@
 #endif
 
 #if HAS_LIBPCI
-# include "pci/pci.h"
+# include <pci/pci.h>
 #endif
 
 #if HAS_LIBPROCPS
 # include <proc/sysinfo.h>
 #endif
 
+#if HAS_LIBGLFW
+# include <GL/gl.h>
+# include <GLFW/glfw3.h>
+#endif
+
 #if HAS_LIBSTATGRAB
 # include <statgrab.h>
 #endif
@@ -240,6 +245,7 @@ static int call_libcpuid_static(Labels *data)
 {
 	int err, i, j = 0;
 	char tmp[MAXSTR * 2] = "";
+	const char *fmt_cpuid    = opts->cpuid_decimal ? "%i" : "0x%X";
 	const char *fmt_cache_kb = _("%d x %d %s, %d-way");
 	const char *fmt_cache_mb = _("%d %s, %d-way");
 	const char *fmt_lines    = _("%s associative, %d-%s line size");
@@ -249,14 +255,13 @@ static int call_libcpuid_static(Labels *data)
 	/* Call libcpuid */
 	MSG_VERBOSE("%s", _("Calling libcpuid for retrieving static data"));
 	if(opts->issue)
-	{
 		cpuid_set_verbosiness_level(2);
-		cpuid_serialize_raw_data(&raw, "");
-	}
 	if(data->l_data->cpuid_raw_file == NULL)
 		err = cpuid_get_raw_data(&raw);
 	else
 		err = cpuid_deserialize_raw_data(&raw, data->l_data->cpuid_raw_file);
+	if(opts->issue)
+		cpuid_serialize_raw_data(&raw, "");
 
 	if(err || cpu_identify(&raw, &datanr))
 	{
@@ -273,15 +278,15 @@ static int call_libcpuid_static(Labels *data)
 		opts->selected_core = 0;
 
 	/* Basically fill CPU tab */
-	casprintf(&data->tab_cpu[VALUE][CODENAME],      false, "%s",  datanr.cpu_codename);
-	casprintf(&data->tab_cpu[VALUE][SPECIFICATION], false, "%s",  datanr.brand_str);
-	casprintf(&data->tab_cpu[VALUE][FAMILY],        false, "0x%X", datanr.family);
-	casprintf(&data->tab_cpu[VALUE][EXTFAMILY],     false, "0x%X", datanr.ext_family);
-	casprintf(&data->tab_cpu[VALUE][MODEL],         false, "0x%X", datanr.model);
-	casprintf(&data->tab_cpu[VALUE][EXTMODEL],      false, "0x%X", datanr.ext_model);
-	casprintf(&data->tab_cpu[VALUE][STEPPING],      false, "%d",  datanr.stepping);
-	casprintf(&data->tab_cpu[VALUE][CORES],         true,  "%d",  datanr.num_cores);
-	casprintf(&data->tab_cpu[VALUE][THREADS],       true,  "%d",  datanr.num_logical_cpus);
+	casprintf(&data->tab_cpu[VALUE][CODENAME],      false, "%s",      datanr.cpu_codename);
+	casprintf(&data->tab_cpu[VALUE][SPECIFICATION], false, "%s",      datanr.brand_str);
+	casprintf(&data->tab_cpu[VALUE][FAMILY],        false, fmt_cpuid, datanr.family);
+	casprintf(&data->tab_cpu[VALUE][EXTFAMILY],     false, fmt_cpuid, datanr.ext_family);
+	casprintf(&data->tab_cpu[VALUE][MODEL],         false, fmt_cpuid, datanr.model);
+	casprintf(&data->tab_cpu[VALUE][EXTMODEL],      false, fmt_cpuid, datanr.ext_model);
+	casprintf(&data->tab_cpu[VALUE][STEPPING],      false, "%d",      datanr.stepping);
+	casprintf(&data->tab_cpu[VALUE][CORES],         true,  "%d",      datanr.num_cores);
+	casprintf(&data->tab_cpu[VALUE][THREADS],       true,  "%d",      datanr.num_logical_cpus);
 
 	/* Improve the CPU Vendor label */
 	const struct CpuVendor { char *standard; char *improved; cpu_vendor_t id; } cpuvendors[] =
@@ -315,26 +320,15 @@ static int call_libcpuid_static(Labels *data)
 
 	/* Cache level 1 (instruction) */
 	if(datanr.l1_instruction_cache > 0)
-	{
-#ifdef HAVE_CPU_ID_L1I_INFO
 		casprintf(&data->tab_cpu[VALUE][LEVEL1I], true, fmt_cache_kb, datanr.num_cores, datanr.l1_instruction_cache, UNIT_KB, datanr.l1_instruction_assoc);
-#else /* HAVE_CPU_ID_L1I_INFO */
-		casprintf(&data->tab_cpu[VALUE][LEVEL1I], true, fmt_cache_kb, datanr.num_cores, datanr.l1_instruction_cache, UNIT_KB, datanr.l1_assoc);
-#endif /* HAVE_CPU_ID_L1I_INFO */
-	}
 
 	/* Cache level 1 (data) */
 	if(datanr.l1_data_cache > 0)
 	{
 		data->cache_count++;
 		data->w_data->size[0] = datanr.l1_data_cache;
-#ifdef HAVE_CPU_ID_L1I_INFO
 		casprintf(&data->tab_cpu[VALUE][LEVEL1D], true, fmt_cache_kb, datanr.num_cores, datanr.l1_data_cache, UNIT_KB, datanr.l1_data_assoc);
 		casprintf(&data->tab_caches[VALUE][L1SIZE], true, fmt_lines, data->tab_cpu[VALUE][LEVEL1D], datanr.l1_data_cacheline, UNIT_B);
-#else /* HAVE_CPU_ID_L1I_INFO */
-		casprintf(&data->tab_cpu[VALUE][LEVEL1D], true, fmt_cache_kb, datanr.num_cores, datanr.l1_data_cache, UNIT_KB, datanr.l1_assoc);
-		casprintf(&data->tab_caches[VALUE][L1SIZE], true, fmt_lines, data->tab_cpu[VALUE][LEVEL1D], datanr.l1_cacheline, UNIT_B);
-#endif /* HAVE_CPU_ID_L1I_INFO */
 	}
 
 	/* Cache level 2 */
@@ -372,7 +366,7 @@ static int call_libcpuid_static(Labels *data)
 
 	/* Add string "HT" in CPU Intructions label (if enabled) */
 	if(datanr.num_cores < datanr.num_logical_cpus)
-		strncat(tmp, "HT", MAXSTR * 2 - strlen(tmp));
+		strncat(tmp, (data->l_data->cpu_vendor_id == VENDOR_INTEL) ? "HT" : "SMT", MAXSTR * 2 - strlen(tmp));
 
 	/* Fill CPU Intructions label */
 	const struct { const cpu_feature_t flag; const char *str; } cpu_flags[] =
@@ -511,51 +505,6 @@ static int cputab_fill_multipliers(Labels *data)
 }
 
 #if HAS_DMIDECODE
-#define TOKEN_LEN 4
-/* Duplicate a string and set unit */
-static char *strdup_and_set_unit(char *str)
-{
-	if(str == NULL)
-		return NULL;
-
-	const ssize_t len = strlen(str) + 1;
-	int i = 0, j = 0;
-	char *ptr = malloc(len);
-	ALLOC_CHECK(ptr);
-
-	while(i < len)
-	{
-		if((str[i] == '@') && (i + TOKEN_LEN - 1 < len) && (str[i + TOKEN_LEN - 1] == '@'))
-		{
-			/* Set unit in destination string */
-			if(!strncmp(&str[i], "@0B@", TOKEN_LEN))
-				j += snprintf(&ptr[j], len, "%s", UNIT_B);
-			else if(!strncmp(&str[i], "@KB@", TOKEN_LEN))
-				j += snprintf(&ptr[j], len, "%s", UNIT_KB);
-			else if(!strncmp(&str[i], "@MB@", TOKEN_LEN))
-				j += snprintf(&ptr[j], len, "%s", UNIT_MB);
-			else if(!strncmp(&str[i], "@GB@", TOKEN_LEN))
-				j += snprintf(&ptr[j], len, "%s", UNIT_GB);
-			else if(!strncmp(&str[i], "@TB@", TOKEN_LEN))
-				j += snprintf(&ptr[j], len, "%s", UNIT_TB);
-			else
-				MSG_ERROR(_("cannot find unit in '%s' string at position %i"), str, i);
-			i += TOKEN_LEN;
-		}
-		else
-		{
-			/* Copy one character */
-			ptr[j] = str[i];
-			i++;
-			j++;
-		}
-	}
-	ptr[j - 1] = '\0';
-
-	return ptr;
-}
-#undef TOKEN_LEN
-
 /* Call Dmidecode through CPU-X but do nothing else */
 int run_dmidecode(void)
 {
@@ -617,7 +566,7 @@ static int cpu_usage(Labels *data)
 	fp = fopen("/proc/stat","r");
 	fscanf(fp,"%*s %li %li %li %li %*s %*s %*s %*s %*s %*s", &new[USER], &new[NICE], &new[SYSTEM], &new[IDLE]);
 	fclose(fp);
-#else
+#else /* __linux__ */
 	size_t len = sizeof(new) * LASTSTAT;
 
 	if(sysctlbyname("kern.cp_time", new, &len, NULL, 0))
@@ -678,15 +627,18 @@ static bool gpu_is_on(char *device_path)
 {
 #ifdef __linux__
 	return !access(format("%s/driver", device_path), F_OK);
-#else
+#else /* __linux__ */
 	UNUSED(device_path);
-	return false;
+	return true;
 #endif /* __linux__ */
 }
 
 /* Find driver name for a device */
 static int find_gpu_device_path(struct pci_dev *dev, char **device_path)
 {
+	int err = 0;
+
+#ifdef __linux__
 	/* Adapted from http://git.kernel.org/cgit/utils/pciutils/pciutils.git/tree/ls-kernel.c */
 	char *base = NULL;
 
@@ -703,14 +655,17 @@ static int find_gpu_device_path(struct pci_dev *dev, char **device_path)
 	}
 
 	casprintf(device_path, false, "%s/devices/%04x:%02x:%02x.%d", base, dev->domain, dev->bus, dev->dev, dev->func);
-	return 0;
+#else /* __linux__ */
+	err = popen_to_str(device_path, "sysctl hw.dri | grep busid | grep %04x:%02x:%02x.%d | cut -d. -f1-3", dev->domain, dev->bus, dev->dev, dev->func);
+#endif /* __linux__ */
+
+	MSG_DEBUG("find_gpu_device_path: device_path=%s", *device_path);
+	return err;
 }
 
-static int find_gpu_driver(char *device_path, char *driver_name, enum EnGpuDrv *gpu_driver)
+static int find_gpu_kernel_driver(char *device_path, char *driver_name, enum EnGpuDrv *gpu_driver)
 {
 	int i;
-	ssize_t n;
-	char buff[MAXSTR] = "";
 	char *drv = NULL, *cmd = NULL;
 	*gpu_driver = GPUDRV_UNKNOWN;
 	const struct { const char *str; const enum EnGpuDrv val; } gpu_drivers[] =
@@ -722,7 +677,7 @@ static int find_gpu_driver(char *device_path, char *driver_name, enum EnGpuDrv *
 		{ "nvidia",   GPUDRV_NVIDIA  },
 		{ "nouveau",  GPUDRV_NOUVEAU },
 		{ "vfio-pci", GPUDRV_VFIO    },
-		{ buff,       GPUDRV_UNKNOWN }
+		{ "",         GPUDRV_UNKNOWN }
 	};
 
 	/* Check GPU state */
@@ -731,6 +686,9 @@ static int find_gpu_driver(char *device_path, char *driver_name, enum EnGpuDrv *
 		MSG_WARNING(_("No kernel driver in use for graphic card at path %s"), device_path);
 		return 1;
 	}
+#ifdef __linux__
+	ssize_t n;
+	char buff[MAXSTR] = "";
 
 	/* Get driver name (as string) */
 	if((n = readlink(format("%s/driver", device_path), buff, MAXSTR)) < 0)
@@ -741,13 +699,22 @@ static int find_gpu_driver(char *device_path, char *driver_name, enum EnGpuDrv *
 	buff[n] = '\0';
 	drv = strrchr(buff, '/');
 	strncpy(driver_name, (drv == NULL) ? buff : drv + 1, MAXSTR);
+#else /* __linux__ */
+	size_t len = MAXSTR;
+
+	if(!sysctlbyname(format("%s.name", device_path), driver_name, &len, NULL, 0))
+	{
+		if((drv = strchr(driver_name, ' ')) != NULL)
+			drv[0] = '\0';
+	}
+#endif /* __linux__ */
 
 	/* Find driver type */
-	for(i = 0; strstr(gpu_drivers[i].str, driver_name) == NULL; i++);
+	for(i = 0; (i != GPUDRV_UNKNOWN - 1) && (strstr(gpu_drivers[i].str, driver_name) == NULL); i++);
 	*gpu_driver = gpu_drivers[i].val;
 	if(*gpu_driver == GPUDRV_UNKNOWN)
 	{
-		MSG_WARNING(_("Your GPU driver is unknown: %s"), buff);
+		MSG_WARNING(_("Your GPU kernel driver is unknown: %s"), driver_name);
 		return 1;
 	}
 
@@ -767,6 +734,74 @@ static int find_gpu_driver(char *device_path, char *driver_name, enum EnGpuDrv *
 	return 0;
 }
 
+static int find_gpu_user_mode_driver(enum EnGpuDrv gpu_driver, char *user_mode_driver)
+{
+	int err = 0;
+
+#if HAS_LIBGLFW
+	const char *description, *gl_ver = NULL, *umd_ver = NULL;
+	GLFWwindow *win = NULL;
+
+	if(glfwInit() == GLFW_FALSE)
+	{
+		err = glfwGetError(&description);
+		goto clean;
+	}
+
+	glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE);
+	if((err = glfwGetError(&description)) != GLFW_NO_ERROR)
+		goto clean;
+
+	win = glfwCreateWindow(640, 480, "", NULL, NULL);
+	if((err = glfwGetError(&description)) != GLFW_NO_ERROR)
+		goto clean;
+
+	glfwMakeContextCurrent(win);
+	if((err = glfwGetError(&description)) != GLFW_NO_ERROR)
+		goto clean;
+
+	gl_ver = (const char*) glGetString(GL_VERSION);
+	if(gl_ver == NULL)
+	{
+		err = glGetError() == GL_NO_ERROR ? -1 : (int) glGetError();
+		description = "glGetString";
+		goto clean;
+	}
+
+	switch(gpu_driver)
+	{
+		case GPUDRV_AMDGPU:
+		case GPUDRV_INTEL:
+		case GPUDRV_RADEON:
+		case GPUDRV_NOUVEAU:
+		case GPUDRV_NOUVEAU_BUMBLEBEE:
+			umd_ver = strstr(gl_ver, "Mesa");
+			break;
+		case GPUDRV_NVIDIA:
+			umd_ver = strstr(gl_ver, "NVIDIA");
+			break;
+		default:
+			break;
+	}
+
+	if(umd_ver != NULL)
+		snprintf(user_mode_driver, MAXSTR, "%s", umd_ver);
+	else
+		MSG_WARNING(_("Your GPU user mode driver is unknown: %s"), gl_ver);
+
+clean:
+	if(err)
+		MSG_ERROR(_("failed to call GLFW (%i): %s"), err, description);
+	if(win != NULL)
+		glfwDestroyWindow(win);
+#else
+	UNUSED(gpu_driver);
+	UNUSED(user_mode_driver);
+#endif /* HAS_LIBGLFW */
+
+	return err;
+}
+
 #define DEVICE_VENDOR_STR(d)  pci_lookup_name(pacc, buff, MAXSTR, PCI_LOOKUP_VENDOR, d->vendor_id, d->device_id)
 #define DEVICE_PRODUCT_STR(d) pci_lookup_name(pacc, buff, MAXSTR, PCI_LOOKUP_DEVICE, d->vendor_id, d->device_id)
 /* Find some PCI devices, like chipset and GPU */
@@ -775,7 +810,7 @@ static int find_devices(Labels *data)
 	/* Adapted from http://git.kernel.org/cgit/utils/pciutils/pciutils.git/tree/example.c */
 	bool chipset_found = false;
 	char *gpu_vendor = NULL;
-	char gpu_driver[MAXSTR] = "", buff[MAXSTR] = "";
+	char gpu_driver[MAXSTR] = "", gpu_umd[MAXSTR] = "", buff[MAXSTR] = "";
 	struct pci_access *pacc;
 	struct pci_dev *dev;
 
@@ -826,10 +861,13 @@ static int find_devices(Labels *data)
 			memset(gpu_driver, 0, MAXSTR);
 			memset(buff,       0, MAXSTR);
 			find_gpu_device_path(dev, &data->g_data->device_path[data->gpu_count]);
-			find_gpu_driver(data->g_data->device_path[data->gpu_count], gpu_driver, &data->g_data->gpu_driver[data->gpu_count]);
+			find_gpu_kernel_driver(data->g_data->device_path[data->gpu_count], gpu_driver, &data->g_data->gpu_driver[data->gpu_count]);
+			find_gpu_user_mode_driver(data->g_data->gpu_driver[data->gpu_count], gpu_umd);
 			casprintf(&data->tab_graphics[VALUE][GPU1VENDOR + data->gpu_count * GPUFIELDS], false, "%s", gpu_vendor);
 			casprintf(&data->tab_graphics[VALUE][GPU1DRIVER + data->gpu_count * GPUFIELDS], false, "%s", gpu_driver);
+			casprintf(&data->tab_graphics[VALUE][GPU1UMD    + data->gpu_count * GPUFIELDS], false, "%s", gpu_umd);
 			casprintf(&data->tab_graphics[VALUE][GPU1MODEL  + data->gpu_count * GPUFIELDS], false, "%s", DEVICE_PRODUCT_STR(dev));
+			casprintf(&data->tab_graphics[VALUE][GPU1DIDRID + data->gpu_count * GPUFIELDS], false, "0x%04X:0x%02X", dev->device_id, pci_read_byte(dev, PCI_REVISION_ID));
 			data->gpu_count++;
 		}
 	}
@@ -840,6 +878,19 @@ static int find_devices(Labels *data)
 	if(!data->gpu_count)
 		MSG_ERROR("%s", _("failed to find graphic card vendor and model"));
 
+#if 0
+	while(data->gpu_count < LASTGRAPHICS / GPUFIELDS)
+	{
+		casprintf(&data->tab_graphics[VALUE][GPU1VENDOR + data->gpu_count * GPUFIELDS], false, "Vendor %u", data->gpu_count);
+		casprintf(&data->tab_graphics[VALUE][GPU1DRIVER + data->gpu_count * GPUFIELDS], false, "Driver %u", data->gpu_count);
+		casprintf(&data->tab_graphics[VALUE][GPU1UMD    + data->gpu_count * GPUFIELDS], false, "UMD %u", data->gpu_count);
+		casprintf(&data->tab_graphics[VALUE][GPU1MODEL  + data->gpu_count * GPUFIELDS], false, "Model %u", data->gpu_count);
+		data->gpu_count++;
+	}
+#endif
+	if(opts->selected_gpu >= data->gpu_count)
+		opts->selected_gpu = 0;
+
 	return !chipset_found + !data->gpu_count;
 }
 #undef DEVICE_VENDOR_STR
@@ -867,6 +918,26 @@ static bool can_access_sys_debug_dri(Labels *data)
 	MSG_DEBUG("can_access_sys_debug_dri() ==> %i", ret);
 	return !ret;
 }
+
+static int convert_pcie_speed_to_gen(char *pcie_speed, uint8_t *pcie_gen)
+{
+	if (!strcmp(pcie_speed,       "2.5 GT/s PCIe"))
+		*pcie_gen = 1;
+	else if (!strcmp(pcie_speed,  "5.0 GT/s PCIe"))
+		*pcie_gen = 2;
+	else if (!strcmp(pcie_speed,  "8.0 GT/s PCIe"))
+		*pcie_gen = 3;
+	else if (!strcmp(pcie_speed, "16.0 GT/s PCIe"))
+		*pcie_gen = 4;
+	else if (!strcmp(pcie_speed, "32.0 GT/s PCIe"))
+		*pcie_gen = 5;
+	else if (!strcmp(pcie_speed, "64.0 GT/s PCIe"))
+		*pcie_gen = 6;
+	else
+		*pcie_gen = 0;
+
+	return 0;
+}
 #endif /* __linux__ */
 
 /* Retrieve GPU temperature and clocks */
@@ -874,10 +945,10 @@ static int gpu_monitoring(Labels *data)
 {
 #ifdef __linux__
 	bool gpu_ok;
-	int ret_drm, ret_hwmon, ret_temp, ret_load, ret_gclk, ret_mclk, ret_vram_used, ret_vram_total, ret_gvolt, ret_gpwr;
-	uint8_t i, card_number, failed_count = 0, fglrx_count = 0, nvidia_count = 0;
+	int ret_drm, ret_hwmon, ret_temp, ret_load, ret_gclk, ret_mclk, ret_vram_used, ret_vram_total, ret_gvolt, ret_gpwr, ret_pcie_max_speed, ret_pcie_max_width, ret_pcie_sta_speed, ret_pcie_sta_width;
+	uint8_t i, card_number, failed_count = 0, fglrx_count = 0, nvidia_count = 0, pcie_max_gen = 0, pcie_sta_gen = 0;
 	long double divisor_temp, divisor_gclk, divisor_mclk, divisor_vram, divisor_gvolt, divisor_gpwr;
-	char *temp = NULL, *gclk = NULL, *mclk = NULL, *load = NULL, *vram_used = NULL, *vram_total = NULL, *gvolt = NULL, *gpwr = NULL;
+	char *temp = NULL, *gclk = NULL, *mclk = NULL, *load = NULL, *vram_used = NULL, *vram_total = NULL, *gvolt = NULL, *gpwr = NULL, *pcie_max_speed = NULL, *pcie_max_width = NULL, *pcie_sta_speed = NULL, *pcie_sta_width = NULL;
 	static bool once_error = true;
 	static char *cached_paths_drm[LASTGRAPHICS / GPUFIELDS] = { NULL };
 	static char *cached_paths_hwmon[LASTGRAPHICS / GPUFIELDS] = { NULL };
@@ -896,6 +967,10 @@ static int gpu_monitoring(Labels *data)
 		ret_vram_total = -1;
 		ret_gvolt      = -1;
 		ret_gpwr       = -1;
+		ret_pcie_max_speed = -1;
+		ret_pcie_max_width = -1;
+		ret_pcie_sta_speed = -1;
+		ret_pcie_sta_width = -1;
 		divisor_temp   = 1.0;
 		divisor_gclk   = 1.0;
 		divisor_mclk   = 1.0;
@@ -906,7 +981,7 @@ static int gpu_monitoring(Labels *data)
 		if(gpu_ok && (data->g_data->gpu_driver[i] == GPUDRV_UNKNOWN))
 		{
 			char gpu_driver[MAXSTR] = "";
-			find_gpu_driver(data->g_data->device_path[i], gpu_driver, &data->g_data->gpu_driver[i]);
+			find_gpu_kernel_driver(data->g_data->device_path[i], gpu_driver, &data->g_data->gpu_driver[i]);
 			casprintf(&data->tab_graphics[VALUE][GPU1DRIVER + i * GPUFIELDS], false, "%s", gpu_driver);
 		}
 		else if(!gpu_ok)
@@ -914,6 +989,8 @@ static int gpu_monitoring(Labels *data)
 			data->g_data->gpu_driver[i] = GPUDRV_UNKNOWN;
 			casprintf(&data->tab_graphics[VALUE][GPU1DRIVER      + i * GPUFIELDS], false, _("None"));
 			casprintf(&data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS], false, "---");
+			casprintf(&data->tab_graphics[VALUE][GPU1PCIE        + i * GPUFIELDS], false, "---");
+			casprintf(&data->tab_graphics[VALUE][GPU1DIDRID      + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], false, "---");
@@ -942,6 +1019,7 @@ static int gpu_monitoring(Labels *data)
 					ret_drm = request_sensor_path(format("%s/drm", data->g_data->device_path[i]), &cached_paths_drm[i], RQT_GPU_DRM);
 				if(ret_drm || (cached_paths_drm[i] == NULL) || (sscanf(cached_paths_drm[i], "/sys/bus/pci/devices/%*x:%*x:%*x.%*d/drm/card%hhu", &card_number) != 1))
 					goto skip_clocks;
+
 				break;
 			default:
 				break;
@@ -964,18 +1042,18 @@ static int gpu_monitoring(Labels *data)
 				ret_vram_total = fopen_to_str(&vram_total, "%s/device/mem_info_vram_total",  cached_paths_drm[i]);
 				ret_gvolt      = fopen_to_str(&gvolt,      "%s/in0_input",                   cached_paths_hwmon[i]);
 				ret_gpwr       = fopen_to_str(&gpwr,       "%s/power1_average",              cached_paths_hwmon[i]);
-				divisor_gclk  = 1e6;
-				divisor_mclk  = 1e6;
-				divisor_vram  = 1 << 20;
-				divisor_gvolt = 1e3;
-				divisor_gpwr  = 1e6;
+				divisor_gclk   = 1e6;
+				divisor_mclk   = 1e6;
+				divisor_vram   = 1 << 20;
+				divisor_gvolt  = 1e3;
+				divisor_gpwr   = 1e6;
 				break;
 			}
 			case GPUDRV_FGLRX:
-				ret_temp  = popen_to_str(&temp, "aticonfig --adapter=%1u --odgt | awk '/Sensor/ { print $5 }'",                       fglrx_count);
-				ret_load  = popen_to_str(&load, "aticonfig --adapter=%1u --odgc | awk '/GPU load/ { sub(\"%\",\"\",$4); print $4 }'", fglrx_count);
-				ret_gclk  = popen_to_str(&gclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $4 }'",               fglrx_count);
-				ret_mclk  = popen_to_str(&mclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $5 }'",               fglrx_count);
+				ret_temp       = popen_to_str(&temp, "aticonfig --adapter=%1u --odgt | awk '/Sensor/ { print $5 }'",                       fglrx_count);
+				ret_load       = popen_to_str(&load, "aticonfig --adapter=%1u --odgc | awk '/GPU load/ { sub(\"%\",\"\",$4); print $4 }'", fglrx_count);
+				ret_gclk       = popen_to_str(&gclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $4 }'",               fglrx_count);
+				ret_mclk       = popen_to_str(&mclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $5 }'",               fglrx_count);
 				ret_vram_used  = -1;
 				ret_vram_total = -1;
 				ret_gvolt      = -1;
@@ -985,7 +1063,7 @@ static int gpu_monitoring(Labels *data)
 			case GPUDRV_INTEL:
 				ret_temp       = -1;
 				ret_load       = -1;
-				ret_gclk  = fopen_to_str(&gclk, "%s/gt_cur_freq_mhz", cached_paths_drm[i]);
+				ret_gclk       = fopen_to_str(&gclk, "%s/gt_cur_freq_mhz", cached_paths_drm[i]);
 				ret_mclk       = -1;
 				ret_vram_used  = -1;
 				ret_vram_total = -1;
@@ -995,30 +1073,31 @@ static int gpu_monitoring(Labels *data)
 			case GPUDRV_RADEON:
 				// ret_temp obtained above
 				ret_load       = -1;
-				ret_gclk  = can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "awk -F '(sclk: | mclk:)' 'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
-				ret_mclk  = can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "awk -F '(mclk: | vddc:)' 'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
+				ret_gclk       = can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "awk -F '(sclk: | mclk:)'  'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
+				ret_mclk       = can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "awk -F '(mclk: | vddc:)'  'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
 				ret_vram_used  = -1;
 				ret_vram_total = -1;
-				ret_gvolt      = -1;
+				ret_gvolt      = can_access_sys_debug_dri(data) ? popen_to_str(&gvolt, "awk -F '(vddc: | vddci:)' 'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
 				ret_gpwr       = -1;
-				divisor_gclk = 100.0;
-				divisor_mclk = 100.0;
+				divisor_gclk   = 100.0;
+				divisor_mclk   = 100.0;
 				break;
 			case GPUDRV_NVIDIA:
 			case GPUDRV_NVIDIA_BUMBLEBEE:
 			{
-				/* Doc: https://nvidia.custhelp.com/app/answers/detail/a_id/3751/~/useful-nvidia-smi-queries */
+				/* Doc: https://nvidia.custhelp.com/app/answers/detail/a_id/3751/~/useful-nvidia-smi-queries
+				        https://briot-jerome.developpez.com/fichiers/blog/nvidia-smi/list.txt */
 				const char *nvidia_cmd_base = (data->g_data->gpu_driver[i] == GPUDRV_NVIDIA_BUMBLEBEE) ? "optirun -b none nvidia-smi -c :8" : "nvidia-smi";
 				const char *nvidia_cmd_args = format("%s --format=csv,noheader,nounits --id=%1u", nvidia_cmd_base, nvidia_count);
 				MSG_DEBUG("gpu_monitoring: nvidia: nvidia_cmd_args=%s", nvidia_cmd_args);
-				ret_temp  = popen_to_str(&temp, "%s --query-gpu=temperature.gpu", nvidia_cmd_args);
-				ret_load  = popen_to_str(&load, "%s --query-gpu=utilization.gpu", nvidia_cmd_args);
-				ret_gclk  = popen_to_str(&gclk, "%s --query-gpu=clocks.gr",       nvidia_cmd_args);
-				ret_mclk  = popen_to_str(&mclk, "%s --query-gpu=clocks.mem",      nvidia_cmd_args);
-				ret_vram_used  = -1;
-				ret_vram_total = -1;
+				ret_temp       = popen_to_str(&temp,       "%s --query-gpu=temperature.gpu", nvidia_cmd_args);
+				ret_load       = popen_to_str(&load,       "%s --query-gpu=utilization.gpu", nvidia_cmd_args);
+				ret_gclk       = popen_to_str(&gclk,       "%s --query-gpu=clocks.gr",       nvidia_cmd_args);
+				ret_mclk       = popen_to_str(&mclk,       "%s --query-gpu=clocks.mem",      nvidia_cmd_args);
+				ret_vram_used  = popen_to_str(&vram_used,  "%s --query-gpu=memory.used",     nvidia_cmd_args);
+				ret_vram_total = popen_to_str(&vram_total, "%s --query-gpu=memory.total",    nvidia_cmd_args);
 				ret_gvolt      = -1;
-				ret_gpwr       = -1;
+				ret_gpwr       = popen_to_str(&gpwr,       "%s --query-gpu=power.draw",      nvidia_cmd_args);;
 				nvidia_count++;
 				break;
 			}
@@ -1029,9 +1108,9 @@ static int gpu_monitoring(Labels *data)
 				int ret_pstate = popen_to_str(&pstate, "grep '*' %1$s/%2$u/pstate || sed -n 1p %1$s/%2$u/pstate ", SYS_DEBUG_DRI, card_number);
 				MSG_DEBUG("gpu_monitoring: nouveau: pstate=%s", pstate);
 				// ret_temp obtained above
-				ret_load  = -1;
-				ret_gclk  = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "echo %s | grep -oP '(?<=core )[^ ]*' | cut -d- -f2", pstate) : -1;
-				ret_mclk  = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "echo %s | grep -oP '(?<=memory )[^ ]*'",             pstate) : -1;
+				ret_load       = -1;
+				ret_gclk       = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "echo %s | grep -oP '(?<=core )[^ ]*' | cut -d- -f2", pstate) : -1;
+				ret_mclk       = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "echo %s | grep -oP '(?<=memory )[^ ]*'",             pstate) : -1;
 				ret_vram_used  = -1;
 				ret_vram_total = -1;
 				ret_gvolt      = -1;
@@ -1045,12 +1124,32 @@ static int gpu_monitoring(Labels *data)
 				continue;
 		}
 
+		// Linux 4.13+
+		const char *max_speed_file = format("%s/device/max_link_speed",     cached_paths_drm[i]);
+		const char *max_width_file = format("%s/device/max_link_width",     cached_paths_drm[i]);
+		const char *sta_speed_file = format("%s/device/current_link_speed", cached_paths_drm[i]);
+		const char *sta_width_file = format("%s/device/current_link_width", cached_paths_drm[i]);
+
+		if(!access(max_speed_file, F_OK) && !access(max_width_file, F_OK) &&
+		   !access(sta_width_file, F_OK) && !access(sta_speed_file, F_OK))
+		{
+			ret_pcie_max_speed = fopen_to_str(&pcie_max_speed, "%s", max_speed_file);
+			ret_pcie_max_width = fopen_to_str(&pcie_max_width, "%s", max_width_file);
+			ret_pcie_sta_speed = fopen_to_str(&pcie_sta_speed, "%s", sta_speed_file);
+			ret_pcie_sta_width = fopen_to_str(&pcie_sta_width, "%s", sta_width_file);
+
+			if(!ret_pcie_max_speed)
+				convert_pcie_speed_to_gen(pcie_max_speed, &pcie_max_gen);
+			if(!ret_pcie_sta_speed)
+				convert_pcie_speed_to_gen(pcie_sta_speed, &pcie_sta_gen);
+		}
+
 		if(!ret_load)
 			casprintf(&data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS], false, "%s%%", load);
 		if(!ret_gclk)
-			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], true, "%.2Lf MHz", strtoull(gclk, NULL, 10) / divisor_gclk);
+			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], true, "%.0Lf MHz", strtoull(gclk, NULL, 10) / divisor_gclk);
 		if(!ret_mclk)
-			casprintf(&data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS], true, "%.2Lf MHz", strtoull(mclk, NULL, 10) / divisor_mclk);
+			casprintf(&data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS], true, "%.0Lf MHz", strtoull(mclk, NULL, 10) / divisor_mclk);
 		if(!ret_vram_used && !ret_vram_total)
 			casprintf(&data->tab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS], true, "%.0Lf %s / %.0Lf %s",
 				strtoull(vram_used,  NULL, 10) / divisor_vram, UNIT_MIB,
@@ -1062,8 +1161,13 @@ static int gpu_monitoring(Labels *data)
 skip_clocks:
 		if(!ret_temp)
 			casprintf(&data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS], true, "%.2Lf°C", strtoull(temp, NULL, 10) / divisor_temp);
+		if(!ret_pcie_max_width && !ret_pcie_sta_width && pcie_sta_gen && pcie_max_gen)
+			casprintf(&data->tab_graphics[VALUE][GPU1PCIE        + i * GPUFIELDS], false,
+				_("Current: PCIe Gen%1dx%d / Max: Gen%1dx%d"),
+				pcie_sta_gen, atoi(pcie_sta_width), pcie_max_gen, atoi(pcie_max_width));
 
-		if(ret_temp && ret_load && ret_gclk && ret_mclk && ret_vram_used && ret_vram_total && ret_gvolt && ret_gpwr)
+		if(ret_temp && ret_load && ret_gclk && ret_mclk && ret_vram_used && ret_vram_total && ret_gvolt && ret_gpwr
+		&& ret_pcie_max_speed && ret_pcie_max_width && ret_pcie_sta_speed && ret_pcie_sta_width)
 			failed_count++;
 
 		FREE(temp);
@@ -1074,6 +1178,10 @@ skip_clocks:
 		FREE(vram_total);
 		FREE(gvolt);
 		FREE(gpwr);
+		FREE(pcie_max_speed);
+		FREE(pcie_max_width);
+		FREE(pcie_sta_speed);
+		FREE(pcie_sta_width);
 	}
 
 	if(once_error && failed_count)
@@ -1111,7 +1219,7 @@ static int system_static(Labels *data)
 	/* Distribution label */
 	err += popen_to_str(&data->tab_system[VALUE][DISTRIBUTION], ". /etc/os-release && echo $PRETTY_NAME");
 
-#else
+#else /* __linux__ */
 	char tmp[MAXSTR];
 	size_t len = sizeof(tmp);
 
@@ -1135,28 +1243,31 @@ static int system_dynamic(Labels *data)
 	time_t uptime_s = 0;
 	struct tm *tm;
 	MemoryData *m_data = data->m_data;
+	static PrefixUnit pu_mem  = { .init = false }, pu_swap = { .init = false };
 
 #if HAS_LIBPROCPS
-	const int div = 1 << 10;
-
 	MSG_VERBOSE("%s", _("Calling libprocps"));
 	/* System uptime */
 	uptime_s = (time_t) uptime(NULL, NULL);
 
 	/* Memory variables */
 	meminfo();
-	m_data->mem_usage[BARUSED]    = kb_main_used    / div;
-	m_data->mem_usage[BARBUFFERS] = kb_main_buffers / div;
-	m_data->mem_usage[BARCACHED]  = kb_main_cached  / div;
-	m_data->mem_usage[BARFREE]    = kb_main_free    / div;
-	m_data->mem_usage[BARSWAP]    = kb_swap_used    / div;
-	m_data->mem_total             = kb_main_total   / div;
-	m_data->swap_total            = kb_swap_total   / div;
+	if(!pu_mem.init || !pu_swap.init)
+	{
+		find_best_prefix(kb_main_total, MULT_K, false, &pu_mem);
+		find_best_prefix(kb_swap_total, MULT_K, false, &pu_swap);
+	}
+	m_data->mem_usage[BARUSED]    = kb_main_used    / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARBUFFERS] = kb_main_buffers / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARCACHED]  = kb_main_cached  / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARFREE]    = kb_main_free    / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARSWAP]    = kb_swap_used    / (long double) pu_swap.divisor;
+	m_data->mem_total             = kb_main_total   / (long double) pu_mem.divisor;
+	m_data->swap_total            = kb_swap_total   / (long double) pu_swap.divisor;
 #endif /* HAS_LIBPROCPS */
 
 #if HAS_LIBSTATGRAB
 	static bool called = false;
-	const int div = 1 << 20;
 	sg_mem_stats *mem; /* Memory labels */
 	sg_swap_stats *swap;
 	sg_host_info *info;
@@ -1176,18 +1287,23 @@ static int system_dynamic(Labels *data)
 	uptime_s = info->uptime;
 
 	/* Memory variables */
-	m_data->mem_usage[BARUSED]    = mem->used   / div;
+	if(!pu_mem.init || !pu_swap.init)
+	{
+		find_best_prefix(mem->total,  MULT_NONE, false, &pu_mem);
+		find_best_prefix(swap->total, MULT_NONE, false, &pu_swap);
+	}
+	m_data->mem_usage[BARUSED]    = mem->used   / (long double) pu_mem.divisor;
 	m_data->mem_usage[BARBUFFERS] = 0;
-	m_data->mem_usage[BARCACHED]  = mem->cache  / div;
-	m_data->mem_usage[BARFREE]    = mem->free   / div;
-	m_data->mem_usage[BARSWAP]    = swap->used  / div;
-	m_data->mem_total             = mem->total  / div;
-	m_data->swap_total            = swap->total / div;
+	m_data->mem_usage[BARCACHED]  = mem->cache  / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARFREE]    = mem->free   / (long double) pu_mem.divisor;
+	m_data->mem_usage[BARSWAP]    = swap->used  / (long double) pu_swap.divisor;
+	m_data->mem_total             = mem->total  / (long double) pu_mem.divisor;
+	m_data->swap_total            = swap->total / (long double) pu_swap.divisor;
 #endif /* HAS_LIBSTATGRAB */
 	/* Memory labels */
 	for(i = USED; i < SWAP; i++)
-		casprintf(&data->tab_system[VALUE][i], false, "%5u %s / %5u %s", m_data->mem_usage[j++], UNIT_MIB, m_data->mem_total, UNIT_MIB);
-	casprintf(&data->tab_system[VALUE][SWAP], false, "%5u %s / %5u %s", m_data->mem_usage[j], UNIT_MIB, m_data->swap_total, UNIT_MIB);
+		casprintf(&data->tab_system[VALUE][i], false, "%3.2Lf %s / %3.2Lf %s", m_data->mem_usage[j++], pu_mem.prefix, m_data->mem_total, pu_mem.prefix);
+	casprintf(&data->tab_system[VALUE][SWAP], false, "%3.2Lf %s / %3.2Lf %s", m_data->mem_usage[j], pu_swap.prefix, m_data->swap_total, pu_swap.prefix);
 
 	/* Uptime label */
 	tm = gmtime(&uptime_s);
@@ -1413,19 +1529,6 @@ static int motherboardtab_fallback(Labels *data)
 	return err;
 }
 
-/* Check is string is empty (e.g. contains only non printable characters) */
-static bool string_is_empty(char *str)
-{
-	int i;
-
-	if(str == NULL)
-		return true;
-
-	for(i = 0; (!isalnum(str[i])) && (str[i] != '\0'); i++);
-
-	return (str[i] == '\0');
-}
-
 /* Retrieve static data if other functions failed */
 static int fallback_mode_static(Labels *data)
 {
@@ -1497,7 +1600,11 @@ static int cputab_temp_fallback(Labels *data)
 	}
 #else /* __linux__ */
 	/* Tested on FreeBSD 12: https://github.com/X0rg/CPU-X/issues/121#issuecomment-575985765 */
-	err = popen_to_str(&data->tab_cpu[VALUE][TEMPERATURE], "sysctl -n dev.cpu.%i.temperature", opts->selected_core);
+	char temp[MAXSTR];
+	size_t len = sizeof(temp);
+
+	if(!(err = sysctlbyname(format("dev.cpu.%i.temperature", opts->selected_core), temp, &len, NULL, 0)))
+		casprintf(&data->tab_cpu[VALUE][TEMPERATURE], true, "%s", temp);
 #endif /* __linux__ */
 
 	if(err)
