@@ -86,27 +86,27 @@
 #define MAXSTR                80       /* Max string */
 #define CACHEFIELDS           2        /* Nb of fields by cache frame */
 #define RAMFIELDS             2        /* Nb of fields by bank */
-#define GPUFIELDS             10        /* Nb of fields by GPU frame */
+#define GPUFIELDS             13       /* Nb of fields by GPU frame */
 #define BENCHFIELDS           2        /* Nb of fields by bench frame */
 
 #define UNIT_B                _("bytes")
 /* SI unit prefixes */
-// TRANSLATORS: kilobyte (10³)
+// TRANSLATORS: initials for kilobyte (10³): please put initials (keep it short)
 #define UNIT_KB               _("kB")
-// TRANSLATORS: megabyte (10⁶)
+// TRANSLATORS: initials for megabyte (10⁶): please put initials (keep it short)
 #define UNIT_MB               _("MB")
-// TRANSLATORS: gigabyte (10⁹)
+// TRANSLATORS: initials for gigabyte (10⁹): please put initials (keep it short)
 #define UNIT_GB               _("GB")
-// TRANSLATORS: terabyte (10¹²)
+// TRANSLATORS: initials for terabyte (10¹²): please put initials (keep it short)
 #define UNIT_TB               _("TB")
 /* Binary unit prefixes  */
-// TRANSLATORS: kibibyte (2¹⁰)
+// TRANSLATORS: initials for kibibyte (2¹⁰): please put initials (keep it short)
 #define UNIT_KIB               _("KiB")
-// TRANSLATORS: mebibyte (2²⁰)
+// TRANSLATORS: initials for mebibyte (2²⁰): please put initials (keep it short)
 #define UNIT_MIB               _("MiB")
-// TRANSLATORS: gibibyte (2³⁰)
+// TRANSLATORS: initials for gibibyte (2³⁰): please put initials (keep it short)
 #define UNIT_GIB               _("GiB")
-// TRANSLATORS: tebibyte (2⁴⁰)
+// TRANSLATORS: initials for tebibyte (2⁴⁰): please put initials (keep it short)
 #define UNIT_TIB               _("TiB")
 
 /* Linux-specific paths definition */
@@ -134,7 +134,7 @@ enum EnObjects
 	FRAMMOTHERBOARD, FRAMBIOS, FRAMCHIPSET,
 	FRAMBANK0, FRAMBANK1, FRAMBANK2, FRAMBANK3, FRAMBANK4, FRAMBANK5, FRAMBANK6, FRAMBANK7,
 	FRAMOPERATINGSYSTEM, FRAMMEMORY,
-	FRAMGPU1, FRAMGPU2, FRAMGPU3, FRAMGPU4,
+	FRAMGPU1, FRAMGPU2, FRAMGPU3, FRAMGPU4, FRAMGPU5, FRAMGPU6, FRAMGPU7, FRAMGPU8, FRAMCARDS,
 	FRAMPRIMESLOW, FRAMPRIMEFAST, FRAMPARAM,
 	FRAMABOUT, FRAMLICENSE,
 	LASTOBJ
@@ -188,10 +188,14 @@ enum EnTabSystemExtra
 
 enum EnTabGraphics
 {
-	GPU1VENDOR, GPU1DRIVER, GPU1MODEL, GPU1TEMPERATURE, GPU1USAGE, GPU1MEMUSED, GPU1CORECLOCK, GPU1MEMCLOCK, GPU1VOLTAGE, GPU1POWERAVG,
-	GPU2VENDOR, GPU2DRIVER, GPU2MODEL, GPU2TEMPERATURE, GPU2USAGE, GPU2MEMUSED, GPU2CORECLOCK, GPU2MEMCLOCK, GPU2VOLTAGE, GPU2POWERAVG,
-	GPU3VENDOR, GPU3DRIVER, GPU3MODEL, GPU3TEMPERATURE, GPU3USAGE, GPU3MEMUSED, GPU3CORECLOCK, GPU3MEMCLOCK, GPU3VOLTAGE, GPU3POWERAVG,
-	GPU4VENDOR, GPU4DRIVER, GPU4MODEL, GPU4TEMPERATURE, GPU4USAGE, GPU4MEMUSED, GPU4CORECLOCK, GPU4MEMCLOCK, GPU4VOLTAGE, GPU4POWERAVG,
+	GPU1VENDOR, GPU1DRIVER, GPU1UMD, GPU1MODEL, GPU1DIDRID, GPU1PCIE, GPU1TEMPERATURE, GPU1USAGE, GPU1VOLTAGE, GPU1POWERAVG, GPU1CORECLOCK, GPU1MEMCLOCK, GPU1MEMUSED,
+	GPU2VENDOR, GPU2DRIVER, GPU2UMD, GPU2MODEL, GPU2DIDRID, GPU2PCIE, GPU2TEMPERATURE, GPU2USAGE, GPU2VOLTAGE, GPU2POWERAVG, GPU2CORECLOCK, GPU2MEMCLOCK, GPU2MEMUSED,
+	GPU3VENDOR, GPU3DRIVER, GPU3UMD, GPU3MODEL, GPU3DIDRID, GPU3PCIE, GPU3TEMPERATURE, GPU3USAGE, GPU3VOLTAGE, GPU3POWERAVG, GPU3CORECLOCK, GPU3MEMCLOCK, GPU3MEMUSED,
+	GPU4VENDOR, GPU4DRIVER, GPU4UMD, GPU4MODEL, GPU4DIDRID, GPU4PCIE, GPU4TEMPERATURE, GPU4USAGE, GPU4VOLTAGE, GPU4POWERAVG, GPU4CORECLOCK, GPU4MEMCLOCK, GPU4MEMUSED,
+	GPU5VENDOR, GPU5DRIVER, GPU5UMD, GPU5MODEL, GPU5DIDRID, GPU5PCIE, GPU5TEMPERATURE, GPU5USAGE, GPU5VOLTAGE, GPU5POWERAVG, GPU5CORECLOCK, GPU5MEMCLOCK, GPU5MEMUSED,
+	GPU6VENDOR, GPU6DRIVER, GPU6UMD, GPU6MODEL, GPU6DIDRID, GPU6PCIE, GPU6TEMPERATURE, GPU6USAGE, GPU6VOLTAGE, GPU6POWERAVG, GPU6CORECLOCK, GPU6MEMCLOCK, GPU6MEMUSED,
+	GPU7VENDOR, GPU7DRIVER, GPU7UMD, GPU7MODEL, GPU7DIDRID, GPU7PCIE, GPU7TEMPERATURE, GPU7USAGE, GPU7VOLTAGE, GPU7POWERAVG, GPU7CORECLOCK, GPU7MEMCLOCK, GPU7MEMUSED,
+	GPU8VENDOR, GPU8DRIVER, GPU8UMD, GPU8MODEL, GPU8DIDRID, GPU8PCIE, GPU8TEMPERATURE, GPU8USAGE, GPU8VOLTAGE, GPU8POWERAVG, GPU8CORECLOCK, GPU8MEMCLOCK, GPU8MEMUSED,
 	LASTGRAPHICS
 };
 
@@ -229,6 +233,15 @@ enum EnOptKeymap
 	LASTKEYMAP
 };
 
+enum EnMultipliers
+{
+	MULT_NONE,
+	MULT_K,
+	MULT_M,
+	MULT_G,
+	MULT_T
+};
+
 typedef struct
 {
 	int8_t  cpu_vendor_id;
@@ -246,9 +259,9 @@ typedef struct
 
 typedef struct
 {
-	uint32_t mem_usage[SWAP - USED + 1];
-	uint32_t mem_total;
-	uint32_t swap_total;
+	long double mem_usage[SWAP - USED + 1];
+	long double mem_total;
+	long double swap_total;
 } MemoryData;
 
 typedef struct
@@ -293,8 +306,8 @@ typedef struct
 
 typedef struct
 {
-	bool     color, verbose, debug, issue, use_network, with_daemon, debug_database, freq_fallback;
-	uint8_t  selected_page, selected_core, bw_test;
+	bool     cpuid_decimal, color, verbose, debug, issue, with_daemon, debug_database, freq_fallback;
+	uint8_t  selected_page, selected_core, bw_test, selected_gpu;
 	uint16_t output_type, refr_time;
 	enum EnOptKeymap keymap;
 } Options;
@@ -306,6 +319,13 @@ typedef struct
 	int  last;
 } Arrays;
 
+typedef struct
+{
+	bool     init;
+	char     *prefix;
+	uint64_t divisor;
+} PrefixUnit;
+
 extern Options *opts;
 
 
@@ -329,6 +349,15 @@ int casprintf(char **str, bool clean_str, const char *fmt, ...);
 /* Return a formatted string */
 char *format(char *str, ...);
 
+/* Divide a number (in bytes) with the appropriate prefix */
+void find_best_prefix(uint64_t value, enum EnMultipliers multiplier, bool use_si_prefixes, PrefixUnit *pu);
+
+/* Duplicate a string and set unit */
+char *strdup_and_set_unit(char *str);
+
+/* Check is string is empty (e.g. contains only non printable characters) */
+bool string_is_empty(char *str);
+
 /* Similar to format(), but string can be colorized */
 char *colorized_msg(const char *color, const char *str, ...);
 
