@@ -147,15 +147,19 @@ static void labels_setname(Labels *data)
 		asprintf(&data->objects[FRAMGPU1 + j],                   _("Card %i"), j); // Frame label
 		asprintf(&data->tab_graphics[NAME][GPU1VENDOR      + i], _("Vendor"));
 		asprintf(&data->tab_graphics[NAME][GPU1DRIVER      + i], _("Driver"));
+		asprintf(&data->tab_graphics[NAME][GPU1UMD         + i], _("UMD Version"));
 		asprintf(&data->tab_graphics[NAME][GPU1MODEL       + i], _("Model"));
+		asprintf(&data->tab_graphics[NAME][GPU1DIDRID      + i], _("DeviceID"));
+		asprintf(&data->tab_graphics[NAME][GPU1PCIE        + i], _("Interface"));
 		asprintf(&data->tab_graphics[NAME][GPU1TEMPERATURE + i], _("Temperature"));
 		asprintf(&data->tab_graphics[NAME][GPU1USAGE       + i], _("Usage"));
-		asprintf(&data->tab_graphics[NAME][GPU1MEMUSED     + i], _("Memory Used"));
-		asprintf(&data->tab_graphics[NAME][GPU1CORECLOCK   + i], _("GPU clock"));
-		asprintf(&data->tab_graphics[NAME][GPU1MEMCLOCK    + i], _("Memory clock"));
 		asprintf(&data->tab_graphics[NAME][GPU1VOLTAGE     + i], _("Core Voltage"));
 		asprintf(&data->tab_graphics[NAME][GPU1POWERAVG    + i], _("Power Avg"));
+		asprintf(&data->tab_graphics[NAME][GPU1CORECLOCK   + i], _("GPU clock"));
+		asprintf(&data->tab_graphics[NAME][GPU1MEMCLOCK    + i], _("Memory clock"));
+		asprintf(&data->tab_graphics[NAME][GPU1MEMUSED     + i], _("Memory Used"));
 	}
+	asprintf(&data->objects[FRAMCARDS], _("Cards")); // Frame label
 
 	/* Bench tab */
 	asprintf(&data->objects[TABBENCH],              _("Bench")); // Tab label
@@ -251,7 +255,11 @@ static void dump_data(Labels *data)
 		{ NO_GRAPHICS,    GPU1VENDOR,   FRAMGPU1            },
 		{ NO_GRAPHICS,    GPU2VENDOR,   FRAMGPU2            },
 		{ NO_GRAPHICS,    GPU3VENDOR,   FRAMGPU3            },
-		{ NO_GRAPHICS,    GPU4VENDOR,   FRAMGPU4            }
+		{ NO_GRAPHICS,    GPU4VENDOR,   FRAMGPU4            },
+		{ NO_GRAPHICS,    GPU5VENDOR,   FRAMGPU5            },
+		{ NO_GRAPHICS,    GPU6VENDOR,   FRAMGPU6            },
+		{ NO_GRAPHICS,    GPU7VENDOR,   FRAMGPU7            },
+		{ NO_GRAPHICS,    GPU8VENDOR,   FRAMGPU8            }
 	};
 
 	MSG_VERBOSE("%s", _("Dumping data…"));
@@ -280,17 +288,18 @@ static void dump_data(Labels *data)
 /* Free memory after display labels */
 void labels_free(Labels *data)
 {
+	/* Note: data->objects and data->tab_graphics[VALUE] are not freed:
+	 * values inside are still read when user change active graphic card  */
 	int i, j;
 	Arrays arrays[] =
 	{
-		{ data->objects,               NULL,                         LASTOBJ                         },
 		{ data->tab_cpu[NAME],         data->tab_cpu[VALUE],         LASTCPU                         },
 		{ data->tab_caches[NAME],      data->tab_caches[VALUE],      data->cache_count * CACHEFIELDS },
 		{ data->w_data->test_name,     NULL,                         data->w_data->test_count        },
 		{ data->tab_motherboard[NAME], data->tab_motherboard[VALUE], LASTMOTHERBOARD                 },
 		{ data->tab_memory[NAME],      data->tab_memory[VALUE],      data->dimm_count                },
 		{ data->tab_system[NAME],      data->tab_system[VALUE],      LASTSYSTEM                      },
-		{ data->tab_graphics[NAME],    data->tab_graphics[VALUE],    data->gpu_count * GPUFIELDS     },
+		{ data->tab_graphics[NAME],    NULL,                         data->gpu_count * GPUFIELDS     },
 		{ data->tab_bench[NAME],       data->tab_bench[VALUE],       LASTBENCH                       },
 		{ data->tab_about,             NULL,                         LASTABOUT                       },
 		{ NULL,                        NULL,                         0                               }
@@ -324,24 +333,26 @@ static const struct
 	char       *description;
 } cpux_options[] =
 {
-	{ HAS_GTK,         'G', "gtk",       no_argument,       N_("Start graphical user interface (GUI) (default)")           },
-	{ HAS_NCURSES,     'N', "ncurses",   no_argument,       N_("Start text-based user interface (TUI)")                    },
-	{ true,            'D', "dump",      no_argument,       N_("Dump all data on standard output and exit")                },
-	{ HAS_DMIDECODE,   'M', "dmidecode", no_argument,       N_("Run embedded command dmidecode and exit")                  },
-	{ HAS_BANDWIDTH,   'B', "bandwidth", no_argument,       N_("Run embedded command bandwidth and exit")                  },
-	{ true,            'r', "refresh",   required_argument, N_("Set custom time between two refreshes (in seconds)")       },
-	{ true,            't', "tab",       required_argument, N_("Set default tab (integer)")                                },
-	{ HAS_LIBCPUID,    'c', "core",      required_argument, N_("Select CPU core to monitor (integer)")                     },
-	{ HAS_BANDWIDTH,   'b', "cachetest", required_argument, N_("Set custom bandwidth test for CPU caches speed (integer)") },
-	{ true,            'd', "daemon",    no_argument,       N_("Start and connect to daemon")                              },
-	{ true,            'v', "verbose",   no_argument,       N_("Verbose output")                                           },
-	{ true,            'h', "help",      no_argument,       N_("Print help and exit")                                      },
-	{ true,            'V', "version",   no_argument,       N_("Print version and exit")                                   },
-	{ true,              0, "nocolor",   no_argument,       N_("Disable colored output")                                   },
-	{ true,              0, "debug",     no_argument,       N_("Print information for debugging")                          },
-	{ true,              0, "issue-fmt", no_argument,       N_("Print required information to paste in an issue")          },
-	{ HAS_NCURSES,       0, "keymap",    required_argument, N_("Set key mapping for NCurses mode (a[rrows]|e[macs]|i[nverted-T]|v[im])") },
-	{ true,              0, NULL,        0,                 NULL                                                           }
+	{ HAS_GTK,         'G', "gtk",           no_argument,       N_("Start graphical user interface (GUI) (default)")           },
+	{ HAS_NCURSES,     'N', "ncurses",       no_argument,       N_("Start text-based user interface (TUI)")                    },
+	{ true,            'D', "dump",          no_argument,       N_("Dump all data on standard output and exit")                },
+	{ HAS_DMIDECODE,   'M', "dmidecode",     no_argument,       N_("Run embedded command dmidecode and exit")                  },
+	{ HAS_BANDWIDTH,   'B', "bandwidth",     no_argument,       N_("Run embedded command bandwidth and exit")                  },
+	{ true,            'r', "refresh",       required_argument, N_("Set custom time between two refreshes (in seconds)")       },
+	{ true,            't', "tab",           required_argument, N_("Set default tab (integer)")                                },
+	{ HAS_LIBCPUID,    'c', "core",          required_argument, N_("Select CPU core to monitor (integer)")                     },
+	{ HAS_BANDWIDTH,   'b', "cachetest",     required_argument, N_("Set custom bandwidth test for CPU caches speed (integer)") },
+	{ HAS_LIBPCI,      'g', "gpu",           required_argument, N_("Select default graphic card (integer)")                    },
+	{ true,            'd', "daemon",        no_argument,       N_("Start and connect to daemon")                              },
+	{ true,            'v', "verbose",       no_argument,       N_("Verbose output")                                           },
+	{ true,            'h', "help",          no_argument,       N_("Print help and exit")                                      },
+	{ true,            'V', "version",       no_argument,       N_("Print version and exit")                                   },
+	{ HAS_LIBCPUID,      0, "cpuid-decimal", no_argument,       N_("Print CPUID values in decimal (default is hexadeximal)")   },
+	{ true,              0, "nocolor",       no_argument,       N_("Disable colored output")                                   },
+	{ true,              0, "debug",         no_argument,       N_("Print information for debugging")                          },
+	{ true,              0, "issue-fmt",     no_argument,       N_("Print required information to paste in an issue")          },
+	{ HAS_NCURSES,       0, "keymap",        required_argument, N_("Set key mapping for NCurses mode (a[rrows]|e[macs]|i[nverted-T]|v[im])") },
+	{ true,              0, NULL,            0,                 NULL                                                           }
 };
 
 static const struct
@@ -394,7 +405,7 @@ static void help(char *binary_name)
 
 		if(cpux_options[i].short_opt) snprintf(buff, MAXSTR, "  -%c,", cpux_options[i].short_opt);
 		else                          snprintf(buff, MAXSTR, "     ");
-		MSG_STDOUT("%s --%-10s %s", buff, cpux_options[i].long_opt, _(cpux_options[i].description));
+		MSG_STDOUT("%s --%-14s %s", buff, cpux_options[i].long_opt, _(cpux_options[i].description));
 	}
 
 	MSG_STDOUT("\n%s", _("Influenceable environment variables:"));
@@ -416,6 +427,7 @@ static void version(bool full_header)
 		{ HAS_NCURSES,     "NCURSES",     NCURSES_VERSION     },
 		{ HAS_LIBCPUID,    "LIBCPUID",    LIBCPUID_VERSION    },
 		{ HAS_LIBPCI,      "LIBPCI",      LIBPCI_VERSION      },
+		{ HAS_LIBGLFW,     "LIBGLFW",     LIBGLFW_VERSION     },
 		{ HAS_LIBPROCPS,   "LIBPROCPS",   LIBPROCPS_VERSION   },
 		{ HAS_LIBSTATGRAB, "LIBSTATGRAB", LIBSTATGRAB_VERSION },
 		{ HAS_DMIDECODE,   "DMIDECODE",   DMIDECODE_VERSION   },
@@ -553,6 +565,11 @@ static void parse_arguments(int argc_orig, char *argv_orig[])
 				if(tmp_arg >= 0)
 					opts->bw_test = atoi(optarg);
 				break;
+			case 'g':
+				tmp_arg = atoi(optarg);
+				if(tmp_arg >= 0)
+					opts->selected_gpu = tmp_arg;
+				break;
 			case 'd':
 				opts->with_daemon = true;
 				break;
@@ -568,7 +585,12 @@ static void parse_arguments(int argc_orig, char *argv_orig[])
 				exit(EXIT_SUCCESS);
 				break;
 			case 0:
-				if(!strcmp(longopts[longindex].name, "nocolor"))
+				if(!strcmp(longopts[longindex].name, "cpuid-decimal"))
+				{
+					opts->cpuid_decimal = true;
+					break;
+				}
+				else if(!strcmp(longopts[longindex].name, "nocolor"))
 				{
 					opts->color = false;
 					break;
@@ -584,7 +606,6 @@ static void parse_arguments(int argc_orig, char *argv_orig[])
 					opts->verbose     = true;
 					opts->debug       = true;
 					opts->issue       = true;
-					opts->use_network = 0;
 					opts->output_type = OUT_DUMP;
 					setlocale(LC_ALL, "C");
 					setenv("CPUX_DAEMON_DEBUG", "1", 1);
@@ -750,17 +771,18 @@ int main(int argc, char *argv[])
 	};
 	opts = &(Options)
 	{
+		.cpuid_decimal  = false,
 		.color          = true,
 		.verbose        = false,
 		.debug          = false,
 		.issue          = false,
-		.use_network    = true,
 		.with_daemon    = false,
 		.debug_database = false,
 		.freq_fallback  = false,
 		.selected_page  = 0,
 		.selected_core  = 0,
 		.bw_test        = 0,
+		.selected_gpu   = 0,
 		.refr_time      = 1,
 		.keymap         = ARROWS
 	};
