@@ -119,6 +119,122 @@ char *format(char *str, ...)
 }
 #undef BUFFER_COUNT
 
+/* Divide a number (in bytes) with the appropriate prefix */
+void find_best_prefix(uint64_t value, enum EnMultipliers multiplier, bool use_si_prefixes, PrefixUnit *pu)
+{
+	int i, d;
+	struct Table { enum EnMultipliers multiplier; char *prefix; uint64_t divisor; };
+
+	const struct Table si_prefixes[] =
+	{
+		{ MULT_NONE,  UNIT_B,  1    },
+		{ MULT_K,     UNIT_KB, 1e3  },
+		{ MULT_M,     UNIT_MB, 1e6  },
+		{ MULT_G,     UNIT_GB, 1e9  },
+		{ MULT_T,     UNIT_TB, 1e12 },
+		{ multiplier, "?B",    1    }
+	};
+	const struct Table binary_prefixes[] =
+	{
+		{ MULT_NONE,  UNIT_B,   1ULL       },
+		{ MULT_K,     UNIT_KIB, 1ULL << 10 },
+		{ MULT_M,     UNIT_MIB, 1ULL << 20 },
+		{ MULT_G,     UNIT_GIB, 1ULL << 30 },
+		{ MULT_T,     UNIT_TIB, 1ULL << 40 },
+		{ multiplier, "?iB",    1ULL       }
+	};
+	const struct Table *prefixes = use_si_prefixes ? si_prefixes : binary_prefixes;
+	const ssize_t table_len = (sizeof(si_prefixes) / sizeof(struct Table)) - 1;
+
+	/* Find current multiplier */
+	for(i = 0; prefixes[i].multiplier != multiplier; i++);
+	pu->init    = true;
+	pu->prefix  = prefixes[i].prefix;
+	pu->divisor = prefixes[0].divisor;
+	if(i >= table_len - 1)
+	{
+		/* Due to the loop over an enum, this case is not possible */
+		MSG_ERROR("multiplier=%i, value=%llu", multiplier, value);
+		return;
+	}
+
+	/* Find new prefix and new divisor */
+	for(i = i + 1, d = 1; (i < table_len) && (d < table_len) && ((value / prefixes[d].divisor) > 0); i++, d++)
+	{
+		pu->prefix  = prefixes[i].prefix;
+		pu->divisor = prefixes[d].divisor;
+	}
+}
+
+#define TOKEN_LEN 4
+/* Duplicate a string and set unit */
+char *strdup_and_set_unit(char *str)
+{
+	if(str == NULL)
+		return NULL;
+
+	const ssize_t len = MAXSTR;
+	bool full = false;
+	ssize_t i = 0, j = 0, free = len, written;
+	char *ptr = malloc(len);
+	ALLOC_CHECK(ptr);
+
+	while((i < len) && !full)
+	{
+		if((str[i] == '@') && (i + TOKEN_LEN - 1 < len) && (str[i + TOKEN_LEN - 1] == '@'))
+		{
+			/* Set unit in destination string */
+			if(!strncmp(&str[i], "@0B@", TOKEN_LEN))
+				written = snprintf(&ptr[j], free, "%s", UNIT_B);
+			else if(!strncmp(&str[i], "@KB@", TOKEN_LEN))
+				written = snprintf(&ptr[j], free, "%s", UNIT_KB);
+			else if(!strncmp(&str[i], "@MB@", TOKEN_LEN))
+				written = snprintf(&ptr[j], free, "%s", UNIT_MB);
+			else if(!strncmp(&str[i], "@GB@", TOKEN_LEN))
+				written = snprintf(&ptr[j], free, "%s", UNIT_GB);
+			else if(!strncmp(&str[i], "@TB@", TOKEN_LEN))
+				written = snprintf(&ptr[j], free, "%s", UNIT_TB);
+			else
+				MSG_ERROR(_("cannot find unit in '%s' string at position %i"), str, i);
+			i += TOKEN_LEN;
+		}
+		else
+		{
+			/* Copy one character */
+			ptr[j]  = str[i];
+			written = 1;
+			i      += written;
+		}
+		if((written >= free) && (str[i - 1] != '\0'))
+		{
+			MSG_WARNING(_("String '%s' is too long, truncating…"), str);
+			full = true;
+		}
+		else
+		{
+			j    += written;
+			free -= written;
+		}
+	}
+	ptr[j - 1] = '\0';
+
+	return ptr;
+}
+#undef TOKEN_LEN
+
+/* Check is string is empty (e.g. contains only non printable characters) */
+bool string_is_empty(char *str)
+{
+	int i;
+
+	if(str == NULL)
+		return true;
+
+	for(i = 0; (!isalnum(str[i])) && (str[i] != '\0'); i++);
+
+	return (str[i] == '\0');
+}
+
 /* Similar to format(), but string can be colorized */
 char *colorized_msg(const char *color, const char *str, ...)
 {
