@@ -3,7 +3,7 @@
 # Install packages
 sudo dnf update -y
 sudo dnf group install -y "Fedora Workstation" --allowerasing
-sudo dnf install -y polkit-gnome gnome-tweak-tool virtualbox-guest-additions flatpak-builder flatpak-module-tools
+sudo dnf install -y polkit-gnome gnome-tweak-tool virtualbox-guest-additions
 sudo usermod -a -G wheel,mock vagrant
 
 # Autologin to GDM
@@ -13,14 +13,12 @@ cat | sudo tee "/etc/gdm/custom.conf" << EOF
 AutomaticLogin=vagrant
 AutomaticLoginEnable=True
 EOF
-echo "X-GNOME-Autostart-enabled=false" | sudo tee -a "/etc/xdg/autostart/gnome-initial-setup-first-login.desktop"
 sudo systemctl set-default graphical.target
 
 # Vagrant user settings
 localectl set-keymap "${HOST_LANG//[_.]*/}"
 gsettings set org.gnome.desktop.input-sources sources '[]'
 gsettings set org.gnome.desktop.screensaver lock-enabled false
-
-# Flatpak
-flatpak remote-add --if-not-exists --user flathub https://flathub.org/repo/flathub.flatpakrepo
-flatpak install -y flathub org.gnome.Platform//3.36 org.gnome.Sdk//3.36
+gsettings set org.gnome.desktop.session idle-delay 0
+gsettings set org.gnome.desktop.lockdown disable-lock-screen true
+echo "yes" > "/home/vagrant/.config/gnome-initial-setup-done"
